# Prezentace systému 2023-04

> Úvodní prezentace pro MV, 25.4.2023 ve 13.30.

Prezentace použití systému v testovacím režimu v LS 2022/2023 (B222). Systém
byl nasazen ve 4 předmětech, které jsou různě organizovány. Tato prezentace
shrnuje poznatky z nasazení a používání během první poloviny semestru.

[[_TOC_]]

Zahrnuté předměty a jejich stručné charakteristiky:

- **BI-BLE**: Blender [CS]  
  Grafický předmět, kterého se účastní studenti FIT i mimo FIT. Práce
  v semestru se dělí mezi průběžné domácí úkoly (každý týden) a semestrální
  práci, jejíž téma si volí sami studenti. Výsledky průběžných úkolů jsou
  k dispozici všem studentům v rámci galerie.

- **BI-ZWU**: Základy Webu [CS]  
  Webový předmět, kterého se účastní studenti FIT i mimo FIT. Práce v semestru
  se dělí mezi průběžné úkoly na cvičení/doma (každý týden) a semestrální
  práci. Výsledky studentů jsou z podstaty přístupné na veřejném webu, jelikož
  se jedná o webový předmět.

- **BI-SKJ**: Skriptovací jazyky [CS]  
  Programovací předmět, kterého se účastní studenti FIT se znalostí práce
  s gitem. Práce v semestru se dělí mezi průběžné úkoly na cvičení/doma
  podle tématu a volitelnou semestrální práci. Výsledky studentů se částečně
  vyhodnocují automaticky se zpětnou vazbou studentům.

- **BIE-SKJ**: Scripting Languages [EN]  
  Programovací předmět, kterého se účastní studenti FIT i zahraniční studenti
  bez znalosti práce s gitem. Práce v semestru se dělí mezi průběžné úkoly na
  cvičení/doma. Výsledky studentů se zcela vyhodnocují automaticky se zpětnou
  vazbou studentům.

Jedná se tedy o předměty s relativně širokou škálu požadavků. V každém předmětu
je možné využívat lekce (class) v různých režimech:

1. **Pouze zadání**, žádné odevzdávání (bez studentských repozitářů)  
   - *jako náhrada `courses`*

1. Zadání a **volný prostor pro odevzdávání** (prázdný studentský repozitář)  
   - *pro snadné zadání současně se vstupními a pomocnými daty pro
     studenty, aby si je nemuseli stahovat přes web*
   - *zadání s odevzdávání semestrální práce místo současného řešení nad
     gitlabem*

1. Zadání a **předepsané soubory pro odevzdání** (neprázdný studentský
   repozitář)  
   - *pro zadání úkolů, které předpokládá více odevzdávaných souborů*
   - *pro snazší orientaci studenta v umístění a pojmenování souborů a
     lepší orientaci učitele ve studentském řešení*

1. Zadání, předepsané soubory a **operace nad studentskými daty** (neprázdný
   studentský repozitář a definované CI)  
   - *pro automatické vytváření náhledů nad studentskými daty - přehled
     studentů i učitele nad odevzdanými daty*
   - *pro hromadné operace nad studentskými daty, např. generování
     multimediálních galerií studentských prací*
   - *publikace souborů na webu - generování dokumentace nebo projektové
     stránky semestrální práce*

1. Zadání, předepsané soubory a **spouštění studentských řešení** (neprázdný
   studentský repozitář a definované CI)
   - *pro spouštění studentského kódu v předpřipraveném prostření, které tak
     studenti nemusejí mít nainstalované lokálně, např. validátor nebo
     generátor statických webových stránkek*
   - *pro vzdálené testování a ladění studentského kódu*

1. Zadání, předepsané soubory a **evaluace studentských řešení** (neprázdný
   studentský repozitář a definované CI)
   - *pro automatické vyhodnocování studentských řešení*
   - *pro snadnější pochopení zadání studenty*
   - *pro zpětnou vazbu ke studentskému řešení*

> **Technické pozadí**
>
> Všechny předměty provozuji jako tzv. **otevřené**. Tzn. seznam studentů není
> předem (a fixně) daný, ale každý student sám poptává jednotlivé lekce (class).
> To mi umožňuje jednak neřešit seznam studentů (proti KOSu) a také to umožňuje
> zápis předmětu uživatelům mimo FIT (mimo oficiální zápis).
>
> To, že student sám poptává jednotlivé lekce (assign/acquire) mi také,
> jako učiteli, dává rychlý přehled o počtu studentů, kteří si již zadáni
> vyzvedli. Tedy nadále participují v semestru (alespoň formálně).
>
> Také je možné provozovat předměty jako **uzavřené**, tedy definovat seznam
> studentů zapsaných na předmět. Pak pouze/právě tito studenti, pracují nad
> úlohami, které jsou jim automaticky vytvořeny, nebo si je studenti přiřazují
> sami.
>
> Kopie zadání je vždy součástí studentského repozitáře a díky tomu je vždy
> studentovi k dispozici. Zároveň mi to jako učiteli umožňuje zadání v průběhu
> semestru upravovat nebo doplňovat (např. o testy nebo doplňující ukázky).
> Student je vždy o každé změně informován a aktivně ji akceptuje.

**Původně byl systém navržen (a provozován) s myšlenkou na oddělený vývoj úloh
a jejich testování (evaluaci). Tedy, že učitel vytvoří zadání (případně řešení)
úloh a o evaluaci studentských řešení se postará dodavatel systému. Tento
přístup je nadále možný, ale je doplněný o další, nezávislý přístup.
Tedy, že je nyní v systému možné nezávisle vytvářet jak zadání, tak i způsob
evaluace.**

Dále jsou uvedeny případy použití systému v jednotlivých předmětech a
zkušenosti z výuky. Tyto zkušenosti jsou průběžnou zpětnou vazbou tvůrcům
systému. Většina připomínek je obratem řešena.


## BI-BLE: Blender

Předmět je rozdělený na 3 *class*: Průběžné **Úkoly**, z nich generovaná
**Galerie** a **Semestrální práce**. Výsledky studentů jsou čistě grafické.
A to jednak obrázky, tak videa.

### Průběžné úkoly a jejich výsledky

- https://betaboard.wr.cz/bi-ble/tasks
- https://betaboard.wr.cz/bi-ble/b222/tasks/gaierda1
- https://betaboard.wr.cz/bi-ble/b222/tasks/hrubcjan
- https://betaboard.wr.cz/bi-ble/b222/tasks/cherkele

Studenti mají pro průběžné výsledky jeden repozitář, kde jsou předpřipravené
adresáře podle témat a v nich šablony souborů (obrázků i videí). Jedná se o
souhrné zadání pro všechny týdny (témata). Při pohledu na studentský repozitář
je vidět *portfólio* studentských prací v čele se *showreelem* (poslední úkol).
Na první pohled tak studenti vidí přehledně své odevzdané výsledky, případně
co jim ještě chybí odevzdat.

> **Technické pozadí**
>
> Jelikož předmět není programovací, neočekává se po studentech znalost
> gitu. Pro znalé je git a standardní práce s ním pochopitelně k dispozici,
> ale není nezbytně nutná.
>
> V rámci dashboardu je možné nahrávát multimediální soubory (obrázky a
> videa) přímo, bez použití gitu nebo gitlabu. Soubory se uloží na příslušné
> místo v repozitáři.

### Galerie průběžných studentských prací

- https://betaboard.wr.cz/bi-ble/gallery

Z průběžných studentských prací se automaticky vytváří galerie. Toto řešení
zpřístupňuje studentské výsledky kolegům (jinak jsou studentské repozitáře
privátní). Jako důsledek kopie prací také zaručuje dostupnost výsledků i po
ostranění studentského repozitáře (např. pro propagaci předmětu).

Galerii jsem v minulosti musel zakládat a udržovat ručně na `photos.google.com`.
Tímto řešením zcela odpadá tato ruční práce a update je tak mnohem častější.
Také je snazší změnit formu a strukturu galerie (po tématech nebo
po studentech), obrázky a videa dohromady nebo zvlášť, apod.

> **Technické pozadí**
>
> Tato *class* je "read-only" a cyklicky se obnovuje (cron). V podstatě také
> demonstruje hromadné operace nad všemi studentskými repozitáři.
> Koncepčeně existuje možnost hromadného vyhodnocování studentských prací
> (tzv. *collect*).
>
> Zde je místo vyhodnocení použitý skript, který ze studentských repozitářů
> vybírá jejich upravené soubory (odlišné od zadání) a ty pak přeuloží do
> repozitáře galerie a nad nimi sestaví galerii (`README.md`) s odkazy
> na obrázky a videa.

### Semestrální práce

- https://betaboard.wr.cz/bi-ble?class=project

Druhý studentský repozitář představuje místo pro semestrální práci. V něm
studenti průběžně tvoří svou semestrální práci. V souboru `README.md` popisují
jak název a téma práce, tak také průběh tvorby, zdrojové i pracovní soubory
a nakonec také výsledek.

- https://betaboard.wr.cz/bi-ble/b222/project/gaierda1?class=project
- https://betaboard.wr.cz/bi-ble/b222/project/pospij52?class=project


> **Technické pozadí**
>
> Jelikož si téma práce volí sami studenti, očekává se, že si upraví celý
> repozitář ke svému obrazu. Tedy i popis repozitáře si změní. Ten se prioritně
> bere právě ze soubrou `README.md`, jak je běžné v praxi. Vytváří tak správné
> návyky i na pozadí této "neprogramátorské" úlohy.


## BI-ZWU

Předmět se zabývá výukou základů (statického) webu. Aby bylo pro studenty
i učitele možné pohodlně pracovat i kontrolovat výsledky, je nezbytné, aby se
studentské soubory publikovaly na web serveru. To se po nastavení děje
automatizovaně pomocí CI/CD, tzv. `GitLab Pages`.

### Publikace studentských dat na webu (gitlab pages)

- https://betaboard.wr.cz/bi-zwu/b222/css-2/vascamat?class=css-2
- https://bi-zwu.pages.wr.cz/b222/css-2/vascamat

Nad úlohami neprobíhá automatická evaluace (ale je principiálně možná).
Průběžné úkoly i semestrální práce jsou na stejné (technologické) úrovni.

Každá úloha má v předmětu svůj repozitář. V rámci nich studenti mají za úkol
upravovat předepsané soubory. Je však možné, že budou vytvářet také soubory
vlastní. V některých úlohách je zapnuta možnost publikace.

### Různé způsoby práce

Pro editaci souborů a nahrávání obrázků je možné použít jak dashboard
(pro neznalé gitu/gitlabu), tak gitlab editor (např. pro markdown strukturované
texty) nebo rovnou git (pro efektivní práci a vývoj). Jelikož někteří studenti
nejsou z FITu, umožňuje to různým uživatelům přístup k předmětu na různé
úrovni.

Z pohledu učitele jsou všechny tyto přístupy rovnocené. Je tedy jedno, na jaké
úrovni student se systémem komunikuje. Ve výsledku jsou všechna data uložená
v repozitářích.

- https://betaboard.wr.cz/bi-zwu/b222/ia-local/formama9?class=ia-local
- https://betaboard.wr.cz/bi-zwu/b222/ia-global/formama9?class=ia-global
- https://betaboard.wr.cz/bi-zwu/b222/ia-local/madermi4?class=ia-local
- https://betaboard.wr.cz/bi-zwu/b222/ia-global/madermi4?class=ia-global

Semestrální práce je pak jedním z úkolů a využívá stejnou infrastrukturu. Tedy,
má nastavené automatické zrcadlení na webserver. Také je možné provádět některé
automatické kontroly semestrální práce, ale momentálně nejsou implementované.

> **Technické pozadí**
>
> V minulosti měl tento předmět dohodu s ICT o zpřístupňování vybraných
> repozitářů na speciálním webu pro tento účel zřízenému. To nyní odpadá,
> neboť je to standarní (volitelnou) součástí systému, kdy se obsah repozitáře
> zrcadlí na webserver. Je to tedy dostupné pro libovolný předmět automaticky.


## BI-SKJ

Předmět je rozdělený na několik témat - skriptovacích jazyků. V každém tématu
je několik úloh, které musejí studenti naprogramovat. Také je k dispozici
repozitář pro volitelnou semestrální práci.

- https://betaboard.wr.cz/bi-skj/b222/project/litosjos?group=b222&class=project

V tomto předmětu je vyzkoušen přístup, kdy jedno téma obsahuje více úkolů.
Vyhodnocení úkolů v jednom tématu probíhá najednou, takže na sebe mohou
navazovat, ale také mohou být nezávislé. Hodnocení může být také komplexní přes
celé téma nebo separátní pro jednotlivé úkoly.

- https://betaboard.wr.cz/bi-skj/b222/shell-intro/barinkl?group=b222
- https://betaboard.wr.cz/bi-skj?group=b222&class=sed&sort=-Task+BK+-Task+CNB+-Task+Shrink+-Task+Template
- https://betaboard.wr.cz/bi-skj?group=b222&class=awk&sort=-Age

### Vyhodnocování řešení

O vyhodnocení studentského řešení informují badge. Toto je standardní cesta,
která se používá v praxi (např. ukončení generování webu předmětu na courses
nebo výsledky testů projektu). Tento přístup ve studentech rozvíjí návyky,
které pak použijí v praxi.

Badge jsou přístupné odkudkoliv a jsou to obrázky, které je možné použít
na libovolném místě, například u nadpisu jednotlivých úloh v rámci zadání.
Běžně se používají v seznamu úloh v rámci předmětu.

Badge  mají typicky číselnou hodnotu. Podle této hodnoty je možné studentské
úlohy řadit. Také je možné řadit úlohy podle času poslední změny nebo stavu
úlohy. Pokud učitel změní zadání (např. přidá testy nebo upraví text), student
pomocí badge vidí, že je k dispozici úprava a může si ji prohlédnout a
akceptovat.

Původně jsem zamýšlel vyzkoušet automatické vyhodnocování úloh pouze pro první
téma (3 úkoly) a zbytek nechat bez vyhodnocování. Studentům se automatické
vyhodnocování líbilo a tak si vyžádali i vyhodnocování ostatních témat.

### Repozitáře pro sestavení image a webové zdroje

Součástí předmětu jsou i 2 repozitáře, které nepředstavují přímo úkoly, ale
jsou pro ně pomocné. Jeden repozitář obsahuje definici systému, na kterém běží
vyhodnocování (docker image) a druhý slouží pro publikaci dat, která mají být
přístupná přes web (`https`, používající identické řešení jako v BI-ZWU).

Gitlab umožňuje vytváření issues (pro studenty i učitele). Přes ně je
možné diskutovat zadání nebo řešení, případně hlásit problémy v zadání. Aby
učitel nemusel studentům ručně vytvářet jednotlivá issues k úlohám, systém
vytvoří studentské issues automaticky na základě označených učitelských
issues. Např. vypracujte úlohu A, vypracujte úlohu B, ... To později usnadní
diskuzi nad studentským řešením k jednotlivým úkolům nebo jeho částem.

> **Technické pozadí**
>
> Jelikož různé předměty mohou vyžadovat různé prostředí (např. různé
> kompilátory nebo nástroje), je možné použít repozitář, které takové prostředí
> bude automaticky sestavovat na základě jednoduché definice, co výsledný
> docker image má obsahovat za programy. V rámci CI tohoto repozitáře vznikne
> požadovaný docker image a zároveň se uloží na gitlabu. Díky tomu je přístupný
> pro vyhodnocování studentských řešení nebo jen práci nad nimi.
>
> Docker image umožňuje také snadnou lokální práci na vytváření, ladění zadání
> učitelem, nebo také evaluaci nebo konzultaci studentských řešení. Pokud je
> repozitář pro image studentům přístupný, mohou jej i oni využivat. Mezi
> hlavní přínosy patří:
>
> - nemusí lokálně instalovat žádné prostředí pro jednotlivé úkoly,
> - nemusí řešit reset prostředí mezi testy nebo ladění kódu,
> - prostředí pro lokální vývoj i vzdálenou evaluaci (na gitlabu) je totožné
>   a typicky jiné než zbytek systému (např. lokalizace nebo filesystem),
> - oddělení prostředí úlohy od zbytku systému, takže nehrozí nechtěná práce
>   s lokálními daty nebo ohrožení integrity systému cizím nebo vyvíjeným kódem,
> - rychlejší a snadnější ladění kódu, než při nahrávání do gitlabu a následným
>   spouštěním CI na serveru.
>
> Pro vytváření issues ve studentských repozitářů stačí, aby učitel vytvořil
> tyto issues ve svém repozitáři a označil je pomocí libovolného štítku, který
> pak uvede v konfiguraci. Podle toho se při vytváření studentských repozitářů
> vytvoří i příslušné issues.


## BIE-SKJ

Tento předmět je jazykovou mutací českého předmětu BI-SKJ. Z něj používá jak
docker image pro vyhodnocování úloh, tak repozitář pro data publikovaná na
webu. Všechny úlohy jsou automaticky vyhodnocovány. Jediný podstatný rozdíl
oproti české mutaci je v tom, že pro každou úlohu je zvláštní repozitář místo
společných repozitářů per téma.

> **Technické pozadí**
>
> Pokud některé předměty sdílí prostředí pro běh a vyhodnocování úloh,
> je možné místo vlastní definice repozitáře využít již existují z jiného
> předmětu, nebo i mimo jakýkoliv předmět. To usnadňuje jejich správu a šetří
> zdroje systému (i lokálního systému).
